@@ -3,16 +3,17 @@
 //! Print a file
 //!
 //! ```rust,no_run
-//! use ashpd::desktop::print::PrintProxy;
-//! use ashpd::WindowIdentifier;
 //! use std::fs::File;
 //!
+//! use ashpd::{desktop::print::PrintProxy, WindowIdentifier};
+//!
 //! async fn run() -> ashpd::Result<()> {
 //!     let connection = zbus::Connection::session().await?;
 //!     let proxy = PrintProxy::new(&connection).await?;
 //!     let identifier = WindowIdentifier::default();
 //!
-//!     let file = File::open("/home/bilelmoussaoui/gitlog.pdf").expect("file to print was not found");
+//!     let file =
+//!         File::open("/home/bilelmoussaoui/gitlog.pdf").expect("file to print was not found");
 //!     let pre_print = proxy
 //!         .prepare_print(
 //!             &identifier,
@@ -23,24 +24,17 @@
 //!         )
 //!         .await?;
 //!     proxy
-//!         .print(
-//!             &identifier,
-//!             "test",
-//!             &file,
-//!             Some(pre_print.token),
-//!             true,
-//!         )
+//!         .print(&identifier, "test", &file, Some(pre_print.token), true)
 //!         .await?;
 //!
 //!     Ok(())
 //! }
 //! ```
 
-use std::os::unix::prelude::AsRawFd;
-use std::{fmt, str::FromStr};
+use std::{fmt, os::unix::prelude::AsRawFd, str::FromStr};
 
-use serde::{Deserialize, Serialize, Serializer};
-use zbus::zvariant::{DeserializeDict, Fd, SerializeDict, Signature, Type};
+use serde::{Deserialize, Serialize};
+use zbus::zvariant::{DeserializeDict, Fd, SerializeDict, Type};
 
 use super::{HandleToken, DESTINATION, PATH};
 use crate::{
@@ -48,7 +42,9 @@ use crate::{
     Error, WindowIdentifier,
 };
 
-#[derive(Debug, Clone, Deserialize, PartialEq, Eq)]
+#[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq, Type)]
+#[zvariant(signature = "s")]
+#[serde(rename_all = "PascalCase")]
 /// The page orientation.
 pub enum Orientation {
     /// Landscape.
@@ -104,33 +100,14 @@ impl FromStr for Orientation {
             "ReverseLandscape" | "reverse_landscape" => Ok(Orientation::ReverseLandscape),
             "ReversePortrait" | "reverse_portrait" => Ok(Orientation::ReversePortrait),
             _ => Err(Error::ParseError(
-                "Failed to parse orientation, invalid value".to_string(),
+                "Failed to parse orientation, invalid value",
             )),
         }
     }
 }
 
-impl Serialize for Orientation {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        match self {
-            Self::Landscape => serializer.serialize_str("landscape"),
-            Self::Portrait => serializer.serialize_str("portrait"),
-            Self::ReverseLandscape => serializer.serialize_str("reverse_landscape"),
-            Self::ReversePortrait => serializer.serialize_str("reverse_portrait"),
-        }
-    }
-}
-
-impl Type for Orientation {
-    fn signature() -> Signature<'static> {
-        String::signature()
-    }
-}
-
-#[derive(Debug, Clone, Deserialize, PartialEq, Eq)]
+#[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq, Type)]
+#[zvariant(signature = "s")]
 /// The print quality.
 pub enum Quality {
     /// Draft quality.
@@ -185,28 +162,11 @@ impl FromStr for Quality {
             "Low" | "low" => Ok(Quality::Low),
             "Normal" | "normal" => Ok(Quality::Normal),
             "High" | "high" => Ok(Quality::High),
-            _ => Err(Error::ParseError(
-                "Failed to parse quality, invalid value".to_string(),
-            )),
+            _ => Err(Error::ParseError("Failed to parse quality, invalid value")),
         }
     }
 }
 
-impl Serialize for Quality {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_str(&self.to_string().to_lowercase())
-    }
-}
-
-impl Type for Quality {
-    fn signature() -> Signature<'static> {
-        String::signature()
-    }
-}
-
 #[derive(SerializeDict, DeserializeDict, Type, Debug, Default)]
 /// Print settings to set in the print dialog.
 #[zvariant(signature = "dict")]
@@ -300,35 +260,35 @@ impl Settings {
     /// Sets the paper name.
     #[must_use]
     pub fn paper_format(mut self, paper_format: &str) -> Self {
-        self.paper_format = Some(paper_format.to_string());
+        self.paper_format = Some(paper_format.to_owned());
         self
     }
 
     /// Sets the paper width.
     #[must_use]
     pub fn paper_width(mut self, paper_width: &str) -> Self {
-        self.paper_width = Some(paper_width.to_string());
+        self.paper_width = Some(paper_width.to_owned());
         self
     }
 
     /// Sets the paper height.
     #[must_use]
     pub fn paper_height(mut self, paper_height: &str) -> Self {
-        self.paper_height = Some(paper_height.to_string());
+        self.paper_height = Some(paper_height.to_owned());
         self
     }
 
     /// Sets the number of copies to print.
     #[must_use]
     pub fn n_copies(mut self, n_copies: &str) -> Self {
-        self.n_copies = Some(n_copies.to_string());
+        self.n_copies = Some(n_copies.to_owned());
         self
     }
 
     /// Sets the default paper source.
     #[must_use]
     pub fn default_source(mut self, default_source: &str) -> Self {
-        self.default_source = Some(default_source.to_string());
+        self.default_source = Some(default_source.to_owned());
         self
     }
 
@@ -342,7 +302,7 @@ impl Settings {
     /// Sets the resolution, both resolution-x & resolution-y.
     #[must_use]
     pub fn resolution(mut self, resolution: &str) -> Self {
-        self.resolution = Some(resolution.to_string());
+        self.resolution = Some(resolution.to_owned());
         self
     }
 
@@ -356,77 +316,77 @@ impl Settings {
     /// Sets the duplex printing mode.
     #[must_use]
     pub fn duplex(mut self, duplex: &str) -> Self {
-        self.duplex = Some(duplex.to_string());
+        self.duplex = Some(duplex.to_owned());
         self
     }
 
     /// Whether to collate copies.
     #[must_use]
     pub fn collate(mut self, collate: &str) -> Self {
-        self.collate = Some(collate.to_string());
+        self.collate = Some(collate.to_owned());
         self
     }
 
     /// Sets whether to reverse the order of the printed pages.
     #[must_use]
     pub fn reverse(mut self, reverse: &str) -> Self {
-        self.reverse = Some(reverse.to_string());
+        self.reverse = Some(reverse.to_owned());
         self
     }
 
     /// Sets the media type.
     #[must_use]
     pub fn media_type(mut self, media_type: &str) -> Self {
-        self.media_type = Some(media_type.to_string());
+        self.media_type = Some(media_type.to_owned());
         self
     }
 
     /// Sets the dithering to use.
     #[must_use]
     pub fn dither(mut self, dither: &str) -> Self {
-        self.dither = Some(dither.to_string());
+        self.dither = Some(dither.to_owned());
         self
     }
 
     /// Sets the page scale in percent.
     #[must_use]
     pub fn scale(mut self, scale: &str) -> Self {
-        self.scale = Some(scale.to_string());
+        self.scale = Some(scale.to_owned());
         self
     }
 
     /// Sets what pages to print, one of all, selection, current or ranges.
     #[must_use]
     pub fn print_pages(mut self, print_pages: &str) -> Self {
-        self.print_pages = Some(print_pages.to_string());
+        self.print_pages = Some(print_pages.to_owned());
         self
     }
 
     /// Sets a list of page ranges, formatted like this: 0-2,4,9-11.
     #[must_use]
     pub fn page_ranges(mut self, page_ranges: &str) -> Self {
-        self.page_ranges = Some(page_ranges.to_string());
+        self.page_ranges = Some(page_ranges.to_owned());
         self
     }
 
     /// Sets what pages to print, one of all, even or odd.
     #[must_use]
     pub fn page_set(mut self, page_set: &str) -> Self {
-        self.page_set = Some(page_set.to_string());
+        self.page_set = Some(page_set.to_owned());
         self
     }
 
     /// Sets the finishings.
     #[must_use]
     pub fn finishings(mut self, finishings: &str) -> Self {
-        self.finishings = Some(finishings.to_string());
+        self.finishings = Some(finishings.to_owned());
         self
     }
 
     /// Sets the number of pages per sheet.
     #[must_use]
     pub fn number_up(mut self, number_up: &str) -> Self {
-        self.number_up = Some(number_up.to_string());
+        self.number_up = Some(number_up.to_owned());
         self
     }
 
@@ -434,56 +394,56 @@ impl Settings {
     /// btlr, btrl.
     #[must_use]
     pub fn number_up_layout(mut self, number_up_layout: &str) -> Self {
-        self.number_up_layout = Some(number_up_layout.to_string());
+        self.number_up_layout = Some(number_up_layout.to_owned());
         self
     }
 
     /// Sets the output bin
     #[must_use]
     pub fn output_bin(mut self, output_bin: &str) -> Self {
-        self.output_bin = Some(output_bin.to_string());
+        self.output_bin = Some(output_bin.to_owned());
         self
     }
 
     /// Sets the horizontal resolution in dpi.
     #[must_use]
     pub fn resolution_x(mut self, resolution_x: &str) -> Self {
-        self.resolution_x = Some(resolution_x.to_string());
+        self.resolution_x = Some(resolution_x.to_owned());
         self
     }
 
     /// Sets the vertical resolution in dpi.
     #[must_use]
     pub fn resolution_y(mut self, resolution_y: &str) -> Self {
-        self.resolution_y = Some(resolution_y.to_string());
+        self.resolution_y = Some(resolution_y.to_owned());
         self
     }
 
     /// Sets the resolution in lines per inch.
     #[must_use]
     pub fn print_lpi(mut self, print_lpi: &str) -> Self {
-        self.print_lpi = Some(print_lpi.to_string());
+        self.print_lpi = Some(print_lpi.to_owned());
         self
     }
 
     /// Sets the print-to-file base name.
     #[must_use]
     pub fn output_basename(mut self, output_basename: &str) -> Self {
-        self.output_basename = Some(output_basename.to_string());
+        self.output_basename = Some(output_basename.to_owned());
         self
     }
 
     /// Sets the print-to-file format, one of PS, PDF, SVG.
     #[must_use]
     pub fn output_file_format(mut self, output_file_format: &str) -> Self {
-        self.output_file_format = Some(output_file_format.to_string());
+        self.output_file_format = Some(output_file_format.to_owned());
         self
     }
 
     /// Sets the print-to-file output uri.
     #[must_use]
     pub fn output_uri(mut self, output_uri: &str) -> Self {
-        self.output_uri = Some(output_uri.to_string());
+        self.output_uri = Some(output_uri.to_owned());
         self
     }
 }
@@ -519,21 +479,21 @@ impl PageSetup {
     /// Sets the ppdname.
     #[must_use]
     pub fn ppdname(mut self, ppdname: &str) -> Self {
-        self.ppdname = Some(ppdname.to_string());
+        self.ppdname = Some(ppdname.to_owned());
         self
     }
 
     /// Sets the name of the page setup.
     #[must_use]
     pub fn name(mut self, name: &str) -> Self {
-        self.name = Some(name.to_string());
+        self.name = Some(name.to_owned());
         self
     }
 
     /// Sets the user visible name of the page setup.
     #[must_use]
     pub fn display_name(mut self, display_name: &str) -> Self {
-        self.display_name = Some(display_name.to_string());
+        self.display_name = Some(display_name.to_owned());
         self
     }
 
